# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ricky Elrod <codeblock@fedoraproject.org>
#
""" Tests for jenkins messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests import Base

from .common import add_doc


class TestJenkinsBuildStart(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build starts.
    """
    expected_title = "jenkins.build.start"
    expected_subti = ("Jenkins project 'fedora-mobile' started building")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.start',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/174/")
    msg = {
        u'i': 1,
        u'timestamp': 1392150202.0,
        u'msg_id': u'2014-2ffb842d-7868-47e8-bfb6-910fb93d9aa9',
        u'topic': u'org.fedoraproject.prod.jenkins.build.start',
        u'msg': {
            u'project': 'fedora-mobile',
            u'build': 174,
        },
    }


class TestJenkinsBuildPassed(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build completes successfully.
    """
    expected_title = "jenkins.build.passed"
    expected_subti = ("Jenkins project 'fedora-mobile' built successfully")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.passed',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/174/")
    msg = {
        u'i': 1,
        u'timestamp': 1392150202.0,
        u'msg_id': u'2014-b035a3d4-078b-4733-9df6-be4f5da911bd',
        u'topic': u'org.fedoraproject.prod.jenkins.build.passed',
        u'msg': {
            u'project': 'fedora-mobile',
            u'took': u'1 min 28 sec',
            u'build': 174,
        },
    }


class TestJenkinsBuildFailed(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build completes with a failure.
    """
    expected_title = "jenkins.build.failed"
    expected_subti = ("Jenkins project 'fedora-mobile' failed to build")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.failed',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/165/")
    msg = {
        u'i': 1,
        u'timestamp': 1390982532.0,
        u'msg_id': u'2014-851eb465-56f2-4f3f-bf8a-2b86c95d1246',
        u'topic': u'org.fedoraproject.prod.jenkins.build.failed',
        u'msg': {
            u'project': 'fedora-mobile',
            u'took': u'9.4 sec',
            u'build': 165,
        },
    }

class TestJenkinsBuildAborted(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build is aborted.
    """
    expected_title = "jenkins.build.aborted"
    expected_subti = ("Jenkins project 'fedora-mobile' build aborted")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.aborted',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/165/")
    msg = {
        u'i': 1,
        u'timestamp': 1390982532.0,
        u'msg_id': u'2014-851eb465-56f2-4f3f-bf8a-2b86c95d1246',
        u'topic': u'org.fedoraproject.prod.jenkins.build.aborted',
        u'msg': {
            u'project': 'fedora-mobile',
            u'took': u'9.4 sec',
            u'build': 165,
        },
    }

class TestJenkinsBuildNotBuilt(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build doesn't actually build (wat?).
    """
    expected_title = "jenkins.build.notbuilt"
    expected_subti = ("Jenkins project 'fedora-mobile' did not build")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.notbuilt',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/165/")
    msg = {
        u'i': 1,
        u'timestamp': 1390982532.0,
        u'msg_id': u'2014-851eb465-56f2-4f3f-bf8a-2b86c95d1246',
        u'topic': u'org.fedoraproject.prod.jenkins.build.notbuilt',
        u'msg': {
            u'project': 'fedora-mobile',
            u'took': u'9.4 sec',
            u'build': 165,
        },
    }

class TestJenkinsBuildUnstable(Base):
    """ `Jenkins <http://jenkins.cloud.fedoraproject.org/>`_ publishes these
    messages when a build completes with warnings.
    """
    expected_title = "jenkins.build.unstable"
    expected_subti = ("Jenkins project 'fedora-mobile' built with warnings")
    expected_secondary_icon = ''
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'jenkins/fedora-mobile/build.unstable',
    ])
    expected_link = (
        "http://jenkins.cloud.fedoraproject.org/job/fedora-mobile/165/")
    msg = {
        u'i': 1,
        u'timestamp': 1390982532.0,
        u'msg_id': u'2014-851eb465-56f2-4f3f-bf8a-2b86c95d1246',
        u'topic': u'org.fedoraproject.prod.jenkins.build.unstable',
        u'msg': {
            u'project': 'fedora-mobile',
            u'took': u'9.4 sec',
            u'build': 165,
        },
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for compose messages """

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestPungiKojiStart(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a compose starts.
    """
    expected_title = "pungi.compose.status.change"
    expected_subti = "pungi-koji compose of Fedora-24-20151208.n.7 started"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "msg": {
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
            "status": "STARTED"
        },
        "timestamp": 1449600728.0,
        "topic": "org.fedoraproject.prod.pungi.compose.status.change"
    }


class TestPungiKojiComplete(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a compose starts.
    """
    expected_title = "pungi.compose.status.change"
    expected_subti = "pungi-koji compose of Fedora-24-20151208.n.7 just finished"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "msg": {
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
            "status": "FINISHED"
        },
        "timestamp": 1449600728.0,
        "topic": "org.fedoraproject.prod.pungi.compose.status.change"
    }


class TestPungiPhaseStart(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when the ``createrepo`` phase of a
    compose starts.
    """
    expected_title = "pungi.compose.phase.start"
    expected_subti = "pungi-koji started the createrepo phase of the " + \
        "Fedora-24-20151208.n.7 compose"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.phase.start",
        "msg": {
            "phase_name": "createrepo",
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }


class TestPungiCreateISOTargets(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when createiso targets are assigned.
    """
    expected_title = "pungi.compose.createiso.targets"
    expected_subti = "pungi-koji assigned 3 createiso targets"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set([
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "armhfp/iso/Fedora-24-20151208.n.7-Server-armhfp-dvd1.iso",
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "x86_64/iso/Fedora-24-20151208.n.7-Server-x86_64-dvd1.iso",
    ])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.createiso.targets",
        "msg": {
            "deliverables": [
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/armhfp/iso/Fedora-24-20151208.n.7-Server-armhfp-dvd1.iso",
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/x86_64/iso/Fedora-24-20151208.n.7-Server-x86_64-dvd1.iso"
            ],
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }


class TestPungiCreateISOWin(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a createiso target completes.
    """
    expected_title = "pungi.compose.createiso.imagedone"
    expected_subti = "pungi-koji finished createiso for " + \
            "Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set([
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
    ])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.createiso.imagedone",
        "msg": {
            "variant": "Server",
            "arch": "i386",
            "file": "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose"
            "/Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }


class TestPungiOstree(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when an ostree phase completed
    succesfully.
    """
    cid = 'f99114401ffce2753bd7cd5401bff056a029bb80474859ab769f68586978248d'
    expected_title = "pungi.compose.ostree"
    expected_subti = "pungi-koji ostree compose Fedora-25-20161002.n.0 " + \
        "produced ostree commit %s for x86_64 " % cid + \
        "fedora-atomic/25/x86_64/docker-host"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//branched/Fedora-25-20161002.n.0"
    expected_icon = "https://apps.fedoraproject.org/img/icons/pungi.png"
    expected_objects = set([
        "rawhide/Fedora-25-20161002/n/0"
    ])
    msg = {
        "i": 1, 
        "timestamp": 1475402944.0, 
        "msg_id": "2016-7e49afa4-c85b-424b-90d8-da36bed006b5", 
        "topic": "org.fedoraproject.prod.pungi.compose.ostree", 
        "msg": {
            "arch": "x86_64", 
            "variant": "Atomic", 
            "commitid": "f99114401ffce2753bd7cd5401bff056a029bb80474859ab769f68586978248d", 
            "location": "http://kojipkgs.fedoraproject.org/compose//branched/Fedora-25-20161002.n.0/compose", 
            "ref": "fedora-atomic/25/x86_64/docker-host", 
            "compose_id": "Fedora-25-20161002.n.0"
        }
    }


class TestFailToCompose(Base):
    """ The `pungi` tool produces these messages when it totally fails to start
    the compose process.
    """
    expected_title = "pungi.compose.fail.to.start"
    expected_subti = \
        "failed to compose from fedora-docker.conf: " + \
        "\"must specify label for a production compose\""
    expected_link = None
    msg = {
        "headers":{},
        "i":1,
        "msg":{
            "command":["/usr/bin/pungi-koji",
                    "--notification-script=/usr/bin/pungi-fedmsg-notification",
                    "--config=fedora-docker.conf",
                    "--old-composes=/mnt/koji/compose/",
                    "--skip-phase=productimg",
                    "--skip-phase=pkgset",
                    "--skip-phase=gather",
                    "--skip-phase=extra_files",
                    "--skip-phase=creatrepo",
                    "--label=",
                    "--target-dir=/mnt/koji/compose/"],
            "config":"fedora-docker.conf",
            "detail":"must specify label for a production compose",
            "target_dir":"/mnt/koji/compose"
        },
        "msg_id":"2017-503fe1ae-a458-49f7-adf6-de32bdf4f856",
        "timestamp":1503692573.0,
        "topic":"org.fedoraproject.prod.pungi.compose.fail.to.start",
        "username":"root"
    }


add_doc(locals())



# This file is part of fedmsg.
# Copyright (C) 2018 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url, email2fas
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.pagure import (
    _get_project, DistGitPagureProcessor)


import fedmsg.meta.base


class RatsProcessor(BaseProcessor):
    topic_prefix_re = 'org\\.fedoraproject\\.(dev|stg|prod)'

    __name__ = "rats"
    __description__ = "Run Another Test System"
    __link__ = "https://pagure.io/rats"
    __docs__ = "https://pagure.io/rats"
    __obj__ = "RATS"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    def link(self, msg, **config):
        if 'test.simple-koji-ci' in msg['topic']:
            msg2 = msg.copy()
            msg2['topic'] = msg2['topic'].replace(
                'rats.test.simple-koji-ci', 'pagure.pull-request.new')
            msg2['msg']['pullrequest'] = msg2['msg']['pull_request']
            return DistGitPagureProcessor(self._, **config).link(msg2, **config)

    def subtitle(self, msg, **config):
        user = msg['msg'].get('agent')

        if 'test.simple-koji-ci' in msg['topic']:
            project = _get_project(msg['msg']['pull_request'])
            prid = prid = msg['msg']['pull_request']['id']
            tmpl = self._(
                '{user} requested a run of simple-koji-ci on '
                '"{project}/pull-request/{prid}"'
            )
            return tmpl.format(user=user, project=project, prid=prid)

        elif 'test.taskotron' in msg['topic']:
            test = msg['msg']['test']
            target = msg['msg']['identifier']
            rdbid = msg['msg']['result_id']
            if target:
                tmpl = self._(
                    '{user} requested a run of taskotron\'s {test} on '
                    '"{target}"'
                )
            else:
                tmpl = self._(
                    '{user} requested a run of taskotron on '
                    'resultsdb\'s result id: "{rdbid}"'
                )
            return tmpl.format(
                user=user, test=test, target=target, rdbid=rdbid)

        elif 'test.atomic-ci' in msg['topic']:
            test = msg['msg']['test']
            target = msg['msg']['identifier']
            rdbid = msg['msg']['result_id']
            if target:
                tmpl = self._(
                    '{user} requested a run of atomic-ci on '
                    '"{target}"'
                )
            else:
                tmpl = self._(
                    '{user} requested a run of atomic-ci on '
                    'resultsdb\'s result id: "{rdbid}"'
                )
            return tmpl.format(
                user=user, test=test, target=target, rdbid=rdbid)

    def secondary_icon(self, msg, **config):
        username = msg['msg'].get('agent')
        if username:
            return avatar_url(username)

    def usernames(self, msg, **config):
        username = msg['msg'].get('agent')
        if username:
            return set([username])
        else:
            return set([])

    def objects(self, msg, **config):
        if 'test.simple-koji-ci' in msg['topic']:
            project = _get_project(msg['msg']['pull_request']).strip()
            prid = msg['msg']['pull_request']['id']
            return set([
                'simple-koji-ci/%s/pull-request/%s' % (project, prid)
            ])
        elif 'test.taskotron' in msg['topic']:
            test = msg['msg']['test']
            target = msg['msg']['identifier']
            rdbid = msg['msg']['result_id']
            if target:
                return set([
                    'taskotron/%s/%s' % (test, target)
                ])
            else:
                return set([
                    'taskotron/resultsdb/%s' % (rdbid)
                ])

        elif 'test.atomic-ci' in msg['topic']:
            test = msg['msg']['test']
            target = msg['msg']['identifier']
            rdbid = msg['msg']['result_id']
            if target:
                return set([
                    'AtomicCI/%s' % (target)
                ])
            else:
                return set([
                    'AtomicCI/resultsdb/%s' % (rdbid)
                ])

        return set([])



import fedmsg.tests.test_meta
import arrow

class TestBodhiConglomerateTestingSamePackageSameUser(
        fedmsg.tests.test_meta.ConglomerateBase):
    originals = [
        {
            "i": 3,
            "msg": {
                "agent": "pghmcfc",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: perl-Devel-CheckBin",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "perl-Devel-CheckBin-0.02-2.fc19",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "perl-Devel-CheckBin",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by pghmcfc. ",
                            "timestamp": 1408636112.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "pghmcfc",
                            "update_title": "perl-Devel-CheckBin-0.02-2.fc19"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636106.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release "
                    "of perl-Devel-CheckBin.",
                    "release": {
                        "dist_tag": "f19",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 19",
                        "name": "F19"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "pghmcfc",
                    "title": "perl-Devel-CheckBin-0.02-2.fc19",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-3db55159-6a93-4659-8eb7-19a24255df9c",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636112.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
        {
            "i": 2,
            "msg": {
                "agent": "pghmcfc",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: perl-Devel-CheckBin",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "perl-Devel-CheckBin-0.02-2.fc20",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "perl-Devel-CheckBin",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by pghmcfc. ",
                            "timestamp": 1408636105.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "pghmcfc",
                            "update_title": "perl-Devel-CheckBin-0.02-2.fc20"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636098.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release of "
                    "perl-Devel-CheckBin.",
                    "release": {
                        "dist_tag": "f20",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 20",
                        "name": "F20"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "pghmcfc",
                    "title": "perl-Devel-CheckBin-0.02-2.fc20",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-8572040e-4456-48cc-b5c2-b457879e82ae",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636105.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
    ]
    expected = [{
        'subtitle': 'pghmcfc submitted 2 perl-Devel-CheckBin '
        'testing updates for F19 and F20',
        'subjective': 'pghmcfc submitted 2 perl-Devel-CheckBin '
        'testing updates for F19 and F20',
        'link': 'https://bodhi.fedoraproject.org/updates/perl-Devel-CheckBin',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        'f9800f2daf8aabc0b33bca9b4033019c74db77b3baca2d9f98cc629147430e20'
        '?s=64&d=retro',

        'start_time': 1408636105.0,
        'end_time': 1408636112.0,
        'timestamp': 1408636108.5,
        'human_time': arrow.get(1408636108.5).humanize(),

        'usernames': set(['pghmcfc']),
        'packages': set(['perl-Devel-CheckBin']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.request.testing']),
        'categories': set(['bodhi']),
        'msg_ids': [
            '2014-3db55159-6a93-4659-8eb7-19a24255df9c',
            '2014-8572040e-4456-48cc-b5c2-b457879e82ae',
        ],
    } ]


class TestBodhiConglomerateTestingSamePackageDifferentUser(
        fedmsg.tests.test_meta.ConglomerateBase):
    originals = [
        {
            "i": 3,
            "msg": {
                "agent": "ralph",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: perl-Devel-CheckBin",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "perl-Devel-CheckBin-0.02-2.fc19",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "perl-Devel-CheckBin",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by ralph. ",
                            "timestamp": 1408636112.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "ralph",
                            "update_title": "perl-Devel-CheckBin-0.02-2.fc19"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636106.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release "
                    "of perl-Devel-CheckBin.",
                    "release": {
                        "dist_tag": "f19",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 19",
                        "name": "F19"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "ralph",
                    "title": "perl-Devel-CheckBin-0.02-2.fc19",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-3db55159-6a93-4659-8eb7-19a24255df9c",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636112.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
        {
            "i": 2,
            "msg": {
                "agent": "pghmcfc",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: perl-Devel-CheckBin",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "perl-Devel-CheckBin-0.02-2.fc20",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "perl-Devel-CheckBin",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by pghmcfc. ",
                            "timestamp": 1408636105.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "pghmcfc",
                            "update_title": "perl-Devel-CheckBin-0.02-2.fc20"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636098.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release of "
                    "perl-Devel-CheckBin.",
                    "release": {
                        "dist_tag": "f20",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 20",
                        "name": "F20"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "pghmcfc",
                    "title": "perl-Devel-CheckBin-0.02-2.fc20",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-8572040e-4456-48cc-b5c2-b457879e82ae",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636105.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
    ]
    expected = [{
        'subtitle': 'pghmcfc and ralph submitted 2 perl-Devel-CheckBin '
        'updates for F19 and F20',
        'subjective': 'pghmcfc and ralph submitted 2 perl-Devel-CheckBin '
        'updates for F19 and F20',
        'link': 'https://bodhi.fedoraproject.org/updates/perl-Devel-CheckBin',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',

        'start_time': 1408636105.0,
        'end_time': 1408636112.0,
        'timestamp': 1408636108.5,
        'human_time': arrow.get(1408636108.5).humanize(),

        'usernames': set(['pghmcfc', 'ralph']),
        'packages': set(['perl-Devel-CheckBin']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.request.testing']),
        'categories': set(['bodhi']),
        'msg_ids': [
            '2014-3db55159-6a93-4659-8eb7-19a24255df9c',
            '2014-8572040e-4456-48cc-b5c2-b457879e82ae',
        ],
    } ]


class TestBodhiConglomerateTestingSameUserDifferentPackage(
        fedmsg.tests.test_meta.ConglomerateBase):
    originals = [
        {
            "i": 3,
            "msg": {
                "agent": "pghmcfc",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: perl-Devel-CheckBin",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "perl-Devel-CheckBin-0.02-2.fc19",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "perl-Devel-CheckBin",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by pghmcfc. ",
                            "timestamp": 1408636112.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "pghmcfc",
                            "update_title": "perl-Devel-CheckBin-0.02-2.fc19"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636106.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release "
                    "of perl-Devel-CheckBin.",
                    "release": {
                        "dist_tag": "f19",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 19",
                        "name": "F19"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "pghmcfc",
                    "title": "perl-Devel-CheckBin-0.02-2.fc19",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-3db55159-6a93-4659-8eb7-19a24255df9c",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636112.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
        {
            "i": 2,
            "msg": {
                "agent": "pghmcfc",
                "update": {
                    "approved": None,
                    "bugs": [
                        {
                            "bz_id": 1130581,
                            "parent": False,
                            "security": False,
                            "title": "Review Request: nethack",
                        }
                    ],
                    "builds": [
                        {
                            "nvr": "nethack-0.02-2.fc20",
                            "package": {
                                "committers": [
                                    "pghmcfc"
                                ],
                                "name": "nethack",
                                "suggest_reboot": False
                            }
                        }
                    ],
                    "close_bugs": False,
                    "comments": [
                        {
                            "anonymous": False,
                            "author": "bodhi",
                            "group": None,
                            "karma": 0,
                            "text": "This update has been submitted for "
                            "testing by pghmcfc. ",
                            "timestamp": 1408636105.0,
                            "update_request": "testing",
                            "update_status": "pending",
                            "update_submitter": "pghmcfc",
                            "update_title": "nethack-0.02-2.fc20"
                        }
                    ],
                    "critpath": False,
                    "critpath_approved": False,
                    "date_modified": None,
                    "date_pushed": None,
                    "date_submitted": 1408636098.0,
                    "karma": 0,
                    "nagged": None,
                    "notes": "This is the first Fedora release of "
                    "nethack.",
                    "release": {
                        "dist_tag": "f20",
                        "id_prefix": "FEDORA",
                        "locked": False,
                        "long_name": "Fedora 20",
                        "name": "F20"
                    },
                    "request": "testing",
                    "stable_karma": 2,
                    "status": "pending",
                    "submitter": "pghmcfc",
                    "title": "nethack-0.02-2.fc20",
                    "type": "newpackage",
                    "unstable_karma": -2,
                    "updateid": None
                }
            },
            "msg_id": "2014-8572040e-4456-48cc-b5c2-b457879e82ae",
            "source_name": "datanommer",
            "source_version": "0.6.4",
            "timestamp": 1408636105.0,
            "topic": "org.fedoraproject.prod.bodhi.update.request.testing"
        },
    ]
    expected = [{
        'subtitle': 'pghmcfc submitted nethack and perl-Devel-CheckBin '
        'updates for F19 and F20',
        'subjective': 'pghmcfc submitted nethack and perl-Devel-CheckBin '
        'updates for F19 and F20',
        'link': 'https://bodhi.fedoraproject.org/users/pghmcfc',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        'f9800f2daf8aabc0b33bca9b4033019c74db77b3baca2d9f98cc629147430e20'
        '?s=64&d=retro',

        'start_time': 1408636105.0,
        'end_time': 1408636112.0,
        'timestamp': 1408636108.5,
        'human_time': arrow.get(1408636108.5).humanize(),

        'usernames': set(['pghmcfc']),
        'packages': set(['perl-Devel-CheckBin', 'nethack']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.request.testing']),
        'categories': set(['bodhi']),
        'msg_ids': [
            '2014-3db55159-6a93-4659-8eb7-19a24255df9c',
            '2014-8572040e-4456-48cc-b5c2-b457879e82ae',
        ],
    } ]



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Martin Krizek <mkrizek@redhat.com>
#           Ralph Bean <rbean@redhat.com>

import logging

import requests

from fedmsg_meta_fedora_infrastructure import BaseProcessor

log = logging.getLogger('fedmsg.meta')


class TaskotronProcessor(BaseProcessor):
    __name__ = "taskotron"
    __description__ = "Framework for automated task execution"
    __link__ = "https://taskotron.fedoraproject.org/"
    __docs__ = "https://docs.qadevel.cloud.fedoraproject.org/libtaskotron/latest/"
    __obj__ = "Automated task results"
    __icon__ = "https://apps.fedoraproject.org/img/icons/taskotron.png"

    def subtitle(self, msg, **config):
        if msg['topic'].endswith('taskotron.result.new'):
            taskname = msg['msg']['task'].get('name', '')
            outcome = msg['msg']['result'].get('outcome', '')
            taskitem = msg['msg']['task'].get('item', '')
            return '%s %s for %s' % (taskname, outcome, taskitem)

    def link(self, msg, **config):
        if msg['topic'].endswith('taskotron.result.new'):
            return msg['msg']['result'].get('log_url', '')

    def objects(self, msg, **config):
        packages = self.packages(msg, **config)
        return set([
            '/'.join([
                msg['msg']['task'].get('name', ''),
                package,
                msg['msg']['result'].get('outcome', '')
            ])
            for package in packages
        ])

    def packages(self, msg, **config):
        type = msg['msg']['task']['type']
        if type == 'koji_build':
            nvr = msg['msg']['task']['item']
            name, version, release = nvr.rsplit('-', 2)
            return set([name])
        elif type == 'bodhi_update':
            alias = msg['msg']['task']['item']
            default_url = 'https://bodhi.fedoraproject.org'
            bodhi_url = config.get('bodhi_url', default_url)
            resp = requests.get(bodhi_url + '/updates/' + alias)
            if not bool(resp):
                log.warn("Failed to talk to bodhi %r %r" % (resp, resp.url))
                return set()
            data = resp.json()
            builds = data['update']['builds']
            nvrs = [build['nvr'] for build in builds]
            packages = [nvr.rsplit('-', 2)[0] for nvr in nvrs]
            return set(packages)

        log.warn('Unhandled taskotron type %r' % type)
        return set()

    def secondary_icon(self, msg, **config):
        packages = self.packages(msg, **config)
        if len(packages) != 1:
            # If it is 0 or greater than 1, just use the taskotron icon.
            return self.__icon__
        else:
            url = 'https://apps.fedoraproject.org/packages/images/icons/%s.png'
            package = list(packages)[0]
            return url % package



import fedmsg.tests.test_meta
import arrow

class TestPkgdbConglomerateCommentSubject(
        fedmsg.tests.test_meta.ConglomerateBase):
    subject = 'lmacken'
    expected = [{
        'categories': set(['ansible']),
        'end_time': 1436292424.0,
        'human_time': arrow.get(1436290771.0714285).humanize(),
        'icon': 'https://apps.fedoraproject.org/img/icons/ansible.png',
        'link': 'http://infrastructure.fedoraproject.org/infra/ansible.git/',
        'packages': set([]),
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/203f6cb95b44b5d38aa21425b066dd522d3e19d8919cf4b339f29e0ea7f03e9b?s=64&d=retro',
        'start_time': 1436290155.0,
        'subtitle': 'lmacken ran the bodhi2.yml and proxies.yml playbooks 7 times',
        'subjective': 'You ran the bodhi2.yml and proxies.yml playbooks 7 times',
        'timestamp': 1436290771.0714285,
        'topics': set(['org.fedoraproject.prod.ansible.playbook.complete',
                       'org.fedoraproject.prod.ansible.playbook.start']),
        'usernames': set(['lmacken']),
    }, {
        'subtitle': 'kevin ran the jenkins-cloud.yml playbook 5 times',
        'subjective': 'kevin ran the jenkins-cloud.yml playbook 5 times',
        'link': 'http://infrastructure.fedoraproject.org/infra/ansible.git/',

        'icon': 'https://apps.fedoraproject.org/img/icons/ansible.png',
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        '1a7d8c43c8b89789a33a3266b0e20be7759a502ff38b74ff724a4db6aa33ede8'
        '?s=64&d=retro',

        'start_time': 1436287486.0,
        'end_time': 1436289694.0,
        'timestamp': 1436288676.6,

        'human_time': arrow.get(1436288676.6).humanize(),

        'usernames': set(['kevin']),
        'packages': set([]),
        'topics': set([
            'org.fedoraproject.prod.ansible.playbook.start',
            'org.fedoraproject.prod.ansible.playbook.complete',
        ]),
        'categories': set(['ansible']),
        'msg_ids': [
            '2014-9e46f5ce-4881-4497-aec4-89cf83332b2d',
            '2014-06ef29dd-adb5-47d8-92aa-f1ead0c4d8f1',
        ],
    }]

    originals = [ {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "results": {
                    "proxy01.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy01.stg.phx2.fedoraproject.org": {
                        "changed": 2,
                        "failures": 0,
                        "ok": 55,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy02.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy03.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy04.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy05.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy06.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy07.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy08.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy09.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy10.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy11.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-2b1e1269-0c96-41e8-9b89-e88f6073f364",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436292424.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-46150020-4a1d-4b8d-a5cd-ab3d97dbeffe",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436292289.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "results": {
                    "proxy01.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy01.stg.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy02.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy03.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy04.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy05.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy06.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy07.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy08.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy09.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy10.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    },
                    "proxy11.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 54,
                        "skipped": 0,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-de60d578-7092-4fc4-af6b-713873bc1e5b",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290963.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-15437abe-3948-45e1-b6f7-7e5d789ecd60",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290844.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "results": {
                    "proxy01.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy01.stg.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy02.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy03.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy04.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy05.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy06.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy07.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy08.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy09.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy10.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    },
                    "proxy11.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-7754e09c-88aa-43c4-9a67-65cf09709db2",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290826.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-1108ff47-65fd-4769-90ec-2d3c7dcb0385",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290755.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/bodhi2.yml",
                "results": {
                    "bodhi02.stg.phx2.fedoraproject.org": {
                        "changed": 2,
                        "failures": 0,
                        "ok": 107,
                        "skipped": 41,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-0d2909e5-ac3e-4515-a6b1-1eee223ee123",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290620.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/bodhi2.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-f5356b7b-4b8c-4a9a-a746-64a3e5ca7fbf",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290558.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/bodhi2.yml",
                "results": {
                    "bodhi02.stg.phx2.fedoraproject.org": {
                        "changed": 6,
                        "failures": 0,
                        "ok": 108,
                        "skipped": 41,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-b9b6eb6b-2768-4a04-85fa-76c318dd3012",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290350.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/bodhi2.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-e5670762-fa5c-49e5-b788-3fa33a6ac2ca",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290291.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "results": {
                    "proxy01.stg.phx2.fedoraproject.org": {
                        "changed": 0,
                        "failures": 0,
                        "ok": 14,
                        "skipped": 1,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-819bcbfe-b9c3-4a09-b546-41249c0068ce",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290275.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-4c68804b-ba66-43a7-a064-cb85cf03d1d0",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290267.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "results": {
                    "proxy01.stg.phx2.fedoraproject.org": {
                        "changed": 2,
                        "failures": 0,
                        "ok": 55,
                        "skipped": 0,
                        "unreachable": 0
                    }
                },
                "userid": "lmacken"
            },
            "msg_id": "2015-5c577688-1682-4a70-a2f1-05f1bef88499",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290178.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/home/fedora/lmacken/ansible/playbooks/groups/proxies.yml",
                "playbook_checksum": False,
                "userid": "lmacken"
            },
            "msg_id": "2015-5045127a-aef8-48ac-b4ea-f22871640ee2",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436290155.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "results": {
                    "209.132.184.137": {
                        "changed": 4,
                        "failures": 0,
                        "ok": 28,
                        "skipped": 25,
                        "unreachable": 0
                    },
                    "209.132.184.153": {
                        "changed": 8,
                        "failures": 0,
                        "ok": 78,
                        "skipped": 20,
                        "unreachable": 0
                    },
                    "209.132.184.165": {
                        "changed": 4,
                        "failures": 0,
                        "ok": 28,
                        "skipped": 25,
                        "unreachable": 0
                    },
                    "209.132.184.209": {
                        "changed": 11,
                        "failures": 0,
                        "ok": 36,
                        "skipped": 17,
                        "unreachable": 0
                    },
                    "jenkins-f22.fedorainfracloud.org": {
                        "changed": 11,
                        "failures": 0,
                        "ok": 32,
                        "skipped": 7,
                        "unreachable": 0
                    }
                },
                "userid": "kevin"
            },
            "msg_id": "2015-0a7d8ad6-0867-4eb9-8a45-92c1ade0f2ed",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436289694.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "playbook_checksum": False,
                "userid": "kevin"
            },
            "msg_id": "2015-6f4cdf28-13af-43d6-9993-e4df7a4417cb",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436289416.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "results": {
                    "jenkins-f22.fedorainfracloud.org": {
                        "changed": 11,
                        "failures": 0,
                        "ok": 32,
                        "skipped": 7,
                        "unreachable": 0
                    }
                },
                "userid": "kevin"
            },
            "msg_id": "2015-10fab449-03b0-4c87-b97c-d6efd9c8daed",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288964.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "playbook_checksum": False,
                "userid": "kevin"
            },
            "msg_id": "2015-9e986ed7-2458-4531-b9e7-98886489bc4c",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288838.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "results": {
                    "jenkins-f22.fedorainfracloud.org": {
                        "changed": 11,
                        "failures": 0,
                        "ok": 32,
                        "skipped": 7,
                        "unreachable": 0
                    }
                },
                "userid": "kevin"
            },
            "msg_id": "2015-85796b67-65fc-49bb-b04f-571aa9b2061d",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288816.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "playbook_checksum": False,
                "userid": "kevin"
            },
            "msg_id": "2015-0a51512d-2882-497b-a5d5-5247d91ba4d9",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288687.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "results": {
                    "jenkins-f22.fedorainfracloud.org": {
                        "changed": 18,
                        "failures": 0,
                        "ok": 33,
                        "skipped": 7,
                        "unreachable": 0
                    }
                },
                "userid": "kevin"
            },
            "msg_id": "2015-10188bc8-a4f2-44b8-b84e-bd077323100f",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288669.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "playbook_checksum": False,
                "userid": "kevin"
            },
            "msg_id": "2015-30f9d339-a556-47a9-ae5a-c05a2ea4f271",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436288540.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        },
        {
            "i": 2,
            "msg": {
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "results": {
                    "jenkins-f22.fedorainfracloud.org": {
                        "changed": 4,
                        "failures": 1,
                        "ok": 19,
                        "skipped": 7,
                        "unreachable": 0
                    }
                },
                "userid": "kevin"
            },
            "msg_id": "2015-284b3e88-797c-4dd0-95af-8f76240c7de3",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436287656.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.complete"
        },
        {
            "i": 1,
            "msg": {
                "check": False,
                "extra_vars": {},
                "inventory": "/srv/web/infra/ansible/inventory/",
                "playbook": "/srv/web/infra/ansible/playbooks/groups/jenkins-cloud.yml",
                "playbook_checksum": False,
                "userid": "kevin"
            },
            "msg_id": "2015-19625863-9b34-496e-a6be-a833f41ffaf0",
            "source_name": "datanommer",
            "source_version": "0.6.5",
            "timestamp": 1436287486.0,
            "topic": "org.fedoraproject.prod.ansible.playbook.start"
        }
    ]



import fedmsg.tests.test_meta

import arrow


class TestCoprConglomerateByUser(
        fedmsg.tests.test_meta.ConglomerateBase):
    expected = [{
        'categories': set(['copr']),
        'end_time': 1436455206.0,
        'human_time': arrow.get(1436455009.7142856).humanize(),
        'icon': 'https://apps.fedoraproject.org/img/icons/copr.png',
        'link': 'https://copr.fedoraproject.org/coprs/logocomune/seafile-client/',
        'packages': set([]),
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        '4638766eef9c176dac0b11adb9d1fd974d20d818a9b1ab9d3f78c7252f67fcf3'
        '?s=64&d=retro',
        'start_time': 1436454907.0,
        'subtitle': 'logocomune kicked off 5 rebuilds of the seafile-client copr',
        'subjective': 'logocomune kicked off 5 rebuilds of the seafile-client copr',
        'timestamp': 1436455009.7142856,
        'topics': set(['org.fedoraproject.prod.copr.build.end',
                        'org.fedoraproject.prod.copr.build.start',
                        'org.fedoraproject.prod.copr.chroot.start']),
        'usernames': set(['logocomune'])},
        {'categories': set(['copr']),
        'end_time': 1436455187.0,
        'human_time': arrow.get(1436455055.66).humanize(),
        'icon': 'https://apps.fedoraproject.org/img/icons/copr.png',
        'link': 'https://copr.fedoraproject.org/coprs/andykimpe/freshplayerplugin/',
        'packages': set([]),
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        '046b0af91b968fa7cc33360e0a5c2ba0fdde525780d45aecd54c902a10a5823b'
        '?s=64&d=retro',
        'start_time': 1436454810.0,
        'subtitle': 'andykimpe kicked off 3 rebuilds of the freshplayerplugin copr',
        'subjective': 'andykimpe kicked off 3 rebuilds of the freshplayerplugin copr',
        'timestamp': 1436455055.6666667,
        'topics': set(['org.fedoraproject.prod.copr.build.end',
                        'org.fedoraproject.prod.copr.build.start',
                        'org.fedoraproject.prod.copr.chroot.start']),
        'usernames': set(['andykimpe'])},
        {'categories': set(['copr']),
        'end_time': 1436455177.0,
        'human_time': arrow.get(1436455031.6).humanize(),
        'icon': 'https://apps.fedoraproject.org/img/icons/copr.png',
        'link': 'https://copr.fedoraproject.org/coprs/avsej/nim-devel/',
        'packages': set([]),
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        '8ae6c08d3b2c200def86e6a408e3f774fc885b6c42da85b550c8fdb32461fbb9'
        '?s=64&d=retro',
        'start_time': 1436454814.0,
        'subtitle': 'avsej kicked off 2 rebuilds of the nim-devel copr',
        'subjective': 'avsej kicked off 2 rebuilds of the nim-devel copr',
        'timestamp': 1436455031.6,
        'topics': set(['org.fedoraproject.prod.copr.build.end',
                        'org.fedoraproject.prod.copr.build.start',
                        'org.fedoraproject.prod.copr.chroot.start']),
        'usernames': set(['avsej'])},
    ]

    originals = [ {
        "i": 7,
        "msg": {
            "build": 103162,
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10140,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "build start: user:logocomune copr:seafile-clientpkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.29  pid:10140",
            "who": "worker-27"
        },
        "msg_id": "2015-2a2b5942-c638-416d-85f4-254c800ead28",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455206.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 8,
        "msg": {
            "build": 103162,
            "chroot": "fedora-22-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10140,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "chroot start: chroot:fedora-22-x86_64 user:logocomunecopr:seafile-client pkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.29  pid:10140",
            "who": "worker-27"
        },
        "msg_id": "2015-0b702c9c-7951-4e40-80ab-946e40842e30",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455206.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 8,
        "msg": {
            "build": 103137,
            "copr": "freshplayerplugin",
            "ip": "172.25.86.32",
            "owner": "andykimpe",
            "pid": 10122,
            "pkg": "chromium-42.0.2311.152-1.el6",
            "user": "andykimpe",
            "what": "build start: user:andykimpe copr:freshplayerpluginpkg: chromium-42.0.2311.152-1.el6 build:103137 ip:172.25.86.32  pid:10122",
            "who": "worker-21"
        },
        "msg_id": "2015-d786cf7e-f7ad-4612-9b28-a987774aab05",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455187.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 9,
        "msg": {
            "build": 103137,
            "chroot": "fedora-rawhide-i386",
            "copr": "freshplayerplugin",
            "ip": "172.25.86.32",
            "owner": "andykimpe",
            "pid": 10122,
            "pkg": "chromium-42.0.2311.152-1.el6",
            "user": "andykimpe",
            "what": "chroot start: chroot:fedora-rawhide-i386 user:andykimpecopr:freshplayerplugin pkg: chromium-42.0.2311.152-1.el6 build:103137 ip:172.25.86.32  pid:10122",
            "who": "worker-21"
        },
        "msg_id": "2015-faf0c84a-b40f-43d0-890e-a33e568469ce",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455187.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 9,
        "msg": {
            "build": 103154,
            "chroot": "fedora-rawhide-i386",
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10076,
            "pkg": "seafile-4.2.7-1.fc20",
            "status": 1,
            "user": "logocomune",
            "version": "4.2.7-1.fc20",
            "what": "build end: user:logocomune copr:seafile-client build:103154  pkg: seafile-4.2.7-1.fc20  version: 4.2.7-1.fc20 ip:172.25.86.29  pid:10076 status:1",
            "who": "worker-6"
        },
        "msg_id": "2015-16295a6a-e05a-4129-97a2-df51916af19e",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455184.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 10,
        "msg": {
            "build": 103168,
            "copr": "nim-devel",
            "ip": "172.25.86.30",
            "owner": "avsej",
            "pid": 10091,
            "pkg": "nim-0.11.3-836.g218c61a.el7.centos",
            "user": "avsej",
            "what": "build start: user:avsej copr:nim-develpkg: nim-0.11.3-836.g218c61a.el7.centos build:103168 ip:172.25.86.30  pid:10091",
            "who": "worker-11"
        },
        "msg_id": "2015-c1fb8f9e-53a6-4adb-8222-624a43c0bdf6",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455177.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 11,
        "msg": {
            "build": 103168,
            "chroot": "fedora-22-x86_64",
            "copr": "nim-devel",
            "ip": "172.25.86.30",
            "owner": "avsej",
            "pid": 10091,
            "pkg": "nim-0.11.3-836.g218c61a.el7.centos",
            "user": "avsej",
            "what": "chroot start: chroot:fedora-22-x86_64 user:avsejcopr:nim-devel pkg: nim-0.11.3-836.g218c61a.el7.centos build:103168 ip:172.25.86.30  pid:10091",
            "who": "worker-11"
        },
        "msg_id": "2015-8e1126fa-6ac3-44a9-a73b-8d5c176889b6",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455177.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 6,
        "msg": {
            "build": 103168,
            "chroot": "fedora-22-i386",
            "copr": "nim-devel",
            "ip": "172.25.86.30",
            "owner": "avsej",
            "pid": 10140,
            "pkg": "nim-0.11.3-836.g218c61a.el7.centos",
            "status": 1,
            "user": "avsej",
            "version": "0.11.3-836.g218c61a.el7.centos",
            "what": "build end: user:avsej copr:nim-devel build:103168  pkg: nim-0.11.3-836.g218c61a.el7.centos  version: 0.11.3-836.g218c61a.el7.centos ip:172.25.86.30  pid:10140 status:1",
            "who": "worker-27"
        },
        "msg_id": "2015-93bb0c96-e0e6-4cf3-a51b-ab545a656d0c",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455176.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 17,
        "msg": {
            "build": 103164,
            "chroot": "epel-6-i386",
            "copr": "freshplayerplugin",
            "ip": "172.25.86.32",
            "owner": "andykimpe",
            "pid": 10085,
            "pkg": "chromiumlibs-glib2-2.36.3-4.el6",
            "status": 0,
            "user": "andykimpe",
            "version": "2.36.3-4.el6",
            "what": "build end: user:andykimpe copr:freshplayerplugin build:103164  pkg: chromiumlibs-glib2-2.36.3-4.el6  version: 2.36.3-4.el6 ip:172.25.86.32  pid:10085 status:0",
            "who": "worker-9"
        },
        "msg_id": "2015-8bb726fb-d01f-4c6a-95cb-8af51a9e74a1",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455158.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 7,
        "msg": {
            "build": 103162,
            "copr": "seafile-client",
            "ip": "172.25.86.33",
            "owner": "logocomune",
            "pid": 10107,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "build start: user:logocomune copr:seafile-clientpkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.33  pid:10107",
            "who": "worker-16"
        },
        "msg_id": "2015-63d7d786-5637-41be-af2b-7df91345f686",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455068.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 8,
        "msg": {
            "build": 103162,
            "chroot": "epel-7-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.33",
            "owner": "logocomune",
            "pid": 10107,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "chroot start: chroot:epel-7-x86_64 user:logocomunecopr:seafile-client pkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.33  pid:10107",
            "who": "worker-16"
        },
        "msg_id": "2015-6516ad8f-e54e-4e54-ab50-519a1c20ee05",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436455068.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 15,
        "msg": {
            "build": 103164,
            "copr": "freshplayerplugin",
            "ip": "172.25.86.32",
            "owner": "andykimpe",
            "pid": 10085,
            "pkg": "chromiumlibs-glib2-2.36.3-4.el6",
            "user": "andykimpe",
            "what": "build start: user:andykimpe copr:freshplayerpluginpkg: chromiumlibs-glib2-2.36.3-4.el6 build:103164 ip:172.25.86.32  pid:10085",
            "who": "worker-9"
        },
        "msg_id": "2015-3398ecef-627c-467a-9acb-b72049394850",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454996.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 16,
        "msg": {
            "build": 103164,
            "chroot": "epel-6-i386",
            "copr": "freshplayerplugin",
            "ip": "172.25.86.32",
            "owner": "andykimpe",
            "pid": 10085,
            "pkg": "chromiumlibs-glib2-2.36.3-4.el6",
            "user": "andykimpe",
            "what": "chroot start: chroot:epel-6-i386 user:andykimpecopr:freshplayerplugin pkg: chromiumlibs-glib2-2.36.3-4.el6 build:103164 ip:172.25.86.32  pid:10085",
            "who": "worker-9"
        },
        "msg_id": "2015-f0460a03-6358-494a-8dd1-0def62fef6d0",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454996.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 13,
        "msg": {
            "build": 103162,
            "copr": "seafile-client",
            "ip": "172.25.86.18",
            "owner": "logocomune",
            "pid": 10143,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "build start: user:logocomune copr:seafile-clientpkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.18  pid:10143",
            "who": "worker-28"
        },
        "msg_id": "2015-273d056f-d806-4893-8969-f9aabf88f2c3",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454961.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 14,
        "msg": {
            "build": 103162,
            "chroot": "fedora-21-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.18",
            "owner": "logocomune",
            "pid": 10143,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "chroot start: chroot:fedora-21-x86_64 user:logocomunecopr:seafile-client pkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.18  pid:10143",
            "who": "worker-28"
        },
        "msg_id": "2015-d1e64ce4-f7c7-4639-ad99-ee391115db85",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454961.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 9,
        "msg": {
            "build": 103161,
            "chroot": "fedora-22-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.18",
            "owner": "logocomune",
            "pid": 10088,
            "pkg": "seafile-client-qt-4.2.7-1.fc20",
            "status": 1,
            "user": "logocomune",
            "version": "4.2.7-1.fc20",
            "what": "build end: user:logocomune copr:seafile-client build:103161  pkg: seafile-client-qt-4.2.7-1.fc20  version: 4.2.7-1.fc20 ip:172.25.86.18  pid:10088 status:1",
            "who": "worker-10"
        },
        "msg_id": "2015-363454be-447b-469a-840d-17074cb22385",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454951.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 7,
        "msg": {
            "build": 103154,
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10076,
            "pkg": "seafile-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "build start: user:logocomune copr:seafile-clientpkg: seafile-4.2.7-1.fc20 build:103154 ip:172.25.86.29  pid:10076",
            "who": "worker-6"
        },
        "msg_id": "2015-a48dc863-6888-4dff-b4c8-e6f750896fc2",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454932.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 8,
        "msg": {
            "build": 103154,
            "chroot": "fedora-rawhide-i386",
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10076,
            "pkg": "seafile-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "chroot start: chroot:fedora-rawhide-i386 user:logocomunecopr:seafile-client pkg: seafile-4.2.7-1.fc20 build:103154 ip:172.25.86.29  pid:10076",
            "who": "worker-6"
        },
        "msg_id": "2015-bb7d7971-875e-4aeb-9bcd-9d4368bef505",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454932.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 12,
        "msg": {
            "build": 103161,
            "chroot": "fedora-20-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.29",
            "owner": "logocomune",
            "pid": 10143,
            "pkg": "seafile-client-qt-4.2.7-1.fc20",
            "status": 1,
            "user": "logocomune",
            "version": "4.2.7-1.fc20",
            "what": "build end: user:logocomune copr:seafile-client build:103161  pkg: seafile-client-qt-4.2.7-1.fc20  version: 4.2.7-1.fc20 ip:172.25.86.29  pid:10143 status:1",
            "who": "worker-28"
        },
        "msg_id": "2015-b97ee951-f950-4a0b-bb16-8320b7a9d80e",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454930.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 13,
        "msg": {
            "build": 103162,
            "copr": "seafile-client",
            "ip": "172.25.86.27",
            "owner": "logocomune",
            "pid": 10079,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "build start: user:logocomune copr:seafile-clientpkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.27  pid:10079",
            "who": "worker-7"
        },
        "msg_id": "2015-c513fb14-a1e2-4ba8-8427-70412b6276b8",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454915.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 14,
        "msg": {
            "build": 103162,
            "chroot": "fedora-22-i386",
            "copr": "seafile-client",
            "ip": "172.25.86.27",
            "owner": "logocomune",
            "pid": 10079,
            "pkg": "seafile-client-qt5-4.2.7-1.fc20",
            "user": "logocomune",
            "what": "chroot start: chroot:fedora-22-i386 user:logocomunecopr:seafile-client pkg: seafile-client-qt5-4.2.7-1.fc20 build:103162 ip:172.25.86.27  pid:10079",
            "who": "worker-7"
        },
        "msg_id": "2015-9eae696a-215a-4c52-9056-42e44261a8c8",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454915.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 14,
        "msg": {
            "build": 103161,
            "chroot": "fedora-rawhide-x86_64",
            "copr": "seafile-client",
            "ip": "172.25.86.27",
            "owner": "logocomune",
            "pid": 10064,
            "pkg": "seafile-client-qt-4.2.7-1.fc20",
            "status": 1,
            "user": "logocomune",
            "version": "4.2.7-1.fc20",
            "what": "build end: user:logocomune copr:seafile-client build:103161  pkg: seafile-client-qt-4.2.7-1.fc20  version: 4.2.7-1.fc20 ip:172.25.86.27  pid:10064 status:1",
            "who": "worker-2"
        },
        "msg_id": "2015-9a9506fd-dfae-4493-b7f8-70faaec320dc",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454907.0,
        "topic": "org.fedoraproject.prod.copr.build.end"
    },
    {
        "i": 4,
        "msg": {
            "build": 103168,
            "copr": "nim-devel",
            "ip": "172.25.86.30",
            "owner": "avsej",
            "pid": 10140,
            "pkg": "nim-0.11.3-836.g218c61a.el7.centos",
            "user": "avsej",
            "what": "build start: user:avsej copr:nim-develpkg: nim-0.11.3-836.g218c61a.el7.centos build:103168 ip:172.25.86.30  pid:10140",
            "who": "worker-27"
        },
        "msg_id": "2015-6e90808a-7108-4ce6-9078-f7909b3f10c8",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454814.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    },
    {
        "i": 5,
        "msg": {
            "build": 103168,
            "chroot": "fedora-22-i386",
            "copr": "nim-devel",
            "ip": "172.25.86.30",
            "owner": "avsej",
            "pid": 10140,
            "pkg": "nim-0.11.3-836.g218c61a.el7.centos",
            "user": "avsej",
            "what": "chroot start: chroot:fedora-22-i386 user:avsejcopr:nim-devel pkg: nim-0.11.3-836.g218c61a.el7.centos build:103168 ip:172.25.86.30  pid:10140",
            "who": "worker-27"
        },
        "msg_id": "2015-d0b37df2-092a-4128-b377-27e87b6c5e51",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454814.0,
        "topic": "org.fedoraproject.prod.copr.chroot.start"
    },
    {
        "i": 10,
        "msg": {
            "build": 103137,
            "copr": "freshplayerplugin",
            "ip": "172.25.86.87",
            "owner": "andykimpe",
            "pid": 10137,
            "pkg": "chromium-42.0.2311.152-1.el6",
            "user": "andykimpe",
            "what": "build start: user:andykimpe copr:freshplayerpluginpkg: chromium-42.0.2311.152-1.el6 build:103137 ip:172.25.86.87  pid:10137",
            "who": "worker-26"
        },
        "msg_id": "2015-4d522fa3-8290-4035-954c-84897f9fdb00",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1436454810.0,
        "topic": "org.fedoraproject.prod.copr.build.start"
    }]



# This file is part of fedmsg.
# Copyright (C) 2013-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

import copy

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url

from fedmsg_meta_fedora_infrastructure.conglomerators.copr import \
        copr as copr_conglomerator

_statuses = {
    0: 'failed',
    1: 'success',
    3: 'running',
    5: 'skipped',
}


_long_template = """Package:  {pkg}
COPR:     {owner}/{copr}
Built by: {user}
Status:   {status}
ID:       {build}

Logs:
  Build:     https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/{build}-{pkg_name}/build.log.gz
  Root:      https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/{build}-{pkg_name}/root.log.gz
  Copr:      https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/build-{build}.log
  Mockchain: https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/{build}-{pkg_name}/mockchain.log.gz
Results:     https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/{build}-{pkg_name}/
Repodata:    https://copr-be.cloud.fedoraproject.org/results/{owner}/{copr}/{chroot}/repodata/
"""


def apply_backwards_compat(msg):
    if msg.get('pkg') and msg.get('version'):
        if msg['version'] not in msg['pkg']:
            msg['pkg'] = msg['pkg'] + '-' + msg['version']

    if 'user' in msg and 'owner' not in msg:
        msg['owner'] = msg['user']

    return msg


class CoprsProcessor(BaseProcessor):
    __name__ = "Copr"
    __description__ = "the Cool Other Package Repositories system"
    __link__ = "https://copr.fedorainfracloud.org"
    __docs__ = "https://fedorahosted.org/copr"
    __obj__ = "Extra Repository Updates"
    __icon__ = "https://apps.fedoraproject.org/img/icons/copr.png"

    conglomerators = [
        copr_conglomerator.ByCopr,
        copr_conglomerator.ByUser,
    ]

    def long_form(self, msg, **config):
        if 'copr.build.end' in msg['topic']:
            kwargs = copy.copy(msg['msg'])

            kwargs = apply_backwards_compat(kwargs)

            kwargs['status'] = _statuses.get(kwargs.get('status'), 'unknown')

            # Zero pad buildid
            kwargs['build'] = '%08d' % (kwargs['build'])

            if '-' not in kwargs['version']:
                # This has happened.
                # Example: 2017-c1e242e3-3a1c-4de9-8d37-61640b5e99fa
                name = '?? WHO KNOWS ??'
            else:
                # Also, extract the name from the NVR.
                name, version, release = kwargs['pkg'].rsplit('-', 2)
            kwargs['pkg_name'] = name

            details = _long_template.format(**kwargs)
            return details

    def subtitle(self, msg, **config):

        # So many changes to this message format over time...
        msg['msg'] = apply_backwards_compat(msg['msg'])

        user = msg['msg'].get('user')
        copr = msg['msg'].get('copr')
        chroot = msg['msg'].get('chroot')
        pkg = msg['msg'].get('pkg')

        status = _statuses.get(msg['msg'].get('status'), 'unknown')

        if 'copr.build.start' in msg['topic']:
            if user:
                tmpl = self._("{user} started a new build of the {copr} copr")
            else:
                tmpl = self._("Automated build of the {copr} copr has been started")
        elif 'copr.build.end' in msg['topic']:
            if user:
                tmpl = self._(
                    "{user}'s {copr} copr build of {pkg} for {chroot} "
                    "finished with '{status}'")
            else:
                tmpl = self._(
                    "Automated {copr} copr build of {pkg} for {chroot} "
                    "finished with '{status}'")
        elif 'copr.chroot.start' in msg['topic']:
            tmpl = self._("{user}'s {copr} copr started a new {chroot} chroot")
        elif 'copr.worker.create' in msg['topic']:
            tmpl = self._("a new worker was created")
        else:
            raise NotImplementedError()

        return tmpl.format(user=user, copr=copr, pkg=pkg,
                           chroot=chroot, status=status)

    def link(self, msg, **config):
        user = msg['msg'].get('owner', msg['msg'].get('user'))
        if user and user.startswith('@'):
            # It's a group
            user = user.replace('@', 'g/')
        copr = msg['msg'].get('copr')
        chroot = msg['msg'].get('chroot', None)
        build = msg['msg'].get('build')

        if 'chroot' in msg['topic']:
            tmpl = ("https://copr-be.cloud.fedoraproject.org/"
                    "results/{user}/{copr}/{chroot}/")
        elif 'build' in msg['topic']:
            tmpl = ("https://copr.fedoraproject.org/"
                    "coprs/{user}/{copr}/build/{build}/")
        else:
            return "https://copr.fedoraproject.org"

        return tmpl.format(user=user, copr=copr, chroot=chroot, build=build)

    def secondary_icon(self, msg, **config):
        if 'user' in msg['msg']:
            return avatar_url(msg['msg']['user'])

    def usernames(self, msg, **config):
        usernames = set()
        if 'user' in msg['msg']:
            usernames.add(msg['msg']['user'])
        if 'owner' in msg['msg']:
            usernames.add(msg['msg']['owner'])
        # Filter out any None values
        return set([u for u in usernames if u])

    def objects(self, msg, **config):
        items = ['coprs']

        if 'copr' in msg['msg']:
            items.append(msg['msg']['copr'])

        items.append('.'.join(msg['topic'].split('.')[-2:]))

        if 'chroot' in msg['topic']:
            items.append(msg['msg']['chroot'])

        return set(['/'.join(items)])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for askbot messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestAskbotRetag(Base):
    """ Messages get emitted on this topic when a question on `Ask Fedora
    <https://ask.fedoraproject.org/questions>`_ changes tags.
    It includes information about what tags the package now has, what question
    the tags are for, and who did the changing.  The ``msg['msg']['tags']``
    field describes which tags changed while the
    ``msg['msg']['thread']['tagnames']`` field describes all tags on the
    thread in question.
    """

    expected_title = "askbot.tag.update"
    expected_subti = "ralph altered tags on askbot question 'some title'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/asldkjfalskdjfalskj',
        'tags/asldkjf',
        'tags/asldkjfalskdjf',
        'threads/2',
    ])
    expected_link = "https://ask.fedoraproject.org/question/2/"
    msg = {
        "i": 1,
        "msg": {
            "topmost_post_id": 2,
            "timestamp": 1359945296.0,
            "agent": "ralph",
            "thread": {
                "tagnames": [
                    "town",
                    "ohok",
                    "asldkjfalskdjfalskj",
                    "asldkjf"
                ],
                "pk": 2,
                "title": "some title"
            },
            "tags": [
                "asldkjfalskdjfalskj",
                "asldkjf",
                "asldkjfalskdjf"
            ]
        },
        "topic": "org.fedoraproject.dev.askbot.tag.update",
        "username": "threebean",
        "timestamp": 1359945296.629136
    }


class TestAskbotNewQuestion(Base):
    """ Messages get emitted on this topic anytime an `Ask Fedora
    <https://ask.fedoraproject.org/questions/>`_ question is updated.
    The snippet we have below is an example of a user posting a brand **new
    question** to `Ask Fedora <https://ask.fedoraproject.org/questions/>`_.
    """
    expected_title = "askbot.post.edit"
    expected_subti = "ralph asked the question 'I have a new question'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/lolol',
        'threads/3',
    ])
    expected_link = "https://ask.fedoraproject.org/question/3/"
    msg = {
        "i": 2,
        "msg": {
            "thread": {
                "tagnames": [
                    "lolol"
                ],
                "pk": 3,
                "title": "I have a new question"
            },
            "topmost_post_id": 3,
            "created": True,
            "timestamp": 1359946267.0,
            "agent": "ralph",
            "newly_mentioned_users": [],
            "diff": "lololol\n\nthis is my entry, I hope that you "
            "like it so very much.\n ...",
            "post": {
                "vote_up_count": 0,
                "text": "lololol\r\n\r\nthis is my entry, I hope that "
                "you like it so very much.",
                "summary": "lololol\n\nthis is my entry, I hope that "
                "you like it so very much.\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 4,
                "post_type": "question"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.edit",
        "username": "threebean",
        "timestamp": 1359946267.401213
    }


class LegacyTestAskbotNewQuestionStg(Base):
    """ Messages get emitted on this topic anytime an `Ask Fedora
    <https://ask.stg.fedoraproject.org/questions/>`_ question is updated.
    The snippet we have below is an example of a user posting a brand **new
    question** to `Ask Fedora <https://ask.stg.fedoraproject.org/questions/>`_.
    """
    expected_title = "askbot.post.edit"
    expected_subti = "ralph asked the question 'I have a new question'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/lolol',
        'threads/3',
    ])
    expected_link = "https://ask.stg.fedoraproject.org/question/3/"
    msg = {
        "i": 2,
        "msg": {
            "thread": {
                "tagnames": [
                    "lolol"
                ],
                "pk": 3,
                "title": "I have a new question"
            },
            "topmost_post_id": 3,
            "created": True,
            "timestamp": 1359946267.0,
            "agent": "ralph",
            "newly_mentioned_users": [],
            "diff": "lololol\n\nthis is my entry, I hope that you "
            "like it so very much.\n ...",
            "post": {
                "vote_up_count": 0,
                "text": "lololol\r\n\r\nthis is my entry, I hope that "
                "you like it so very much.",
                "summary": "lololol\n\nthis is my entry, I hope that "
                "you like it so very much.\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 4,
                "post_type": "question"
            }
        },
        "topic": "org.fedoraproject.stg.askbot.post.edit",
        "username": "threebean",
        "timestamp": 1359946267.401213
    }


class TestAskbotNewAnswer(Base):
    """ Messages get emitted on this topic anytime a question is updated.
    An 'update' includes a new question, a new answer, and a modification
    to either.  *This* example is one of a **new answer** being posted to
    an `Ask Fedora <https://ask.fedoraproject.org/questions/>`_ question.
    """
    expected_title = "askbot.post.edit"
    expected_subti = ("ralph suggested an answer "
                      "to the question 'watwatwatwata'")
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph', 'lmacken'])
    expected_objects = set([
        'tags/cool',
        'threads/1',
    ])
    expected_link = "https://ask.fedoraproject.org/question/1/"
    msg = {
        "i": 1,
        "msg": {
            "thread": {
                "tagnames": [
                    "cool"
                ],
                "pk": 1,
                "title": "watwatwatwata"
            },
            "topmost_post_id": 1,
            "created": True,
            "timestamp": 1359946481.0,
            "agent": "ralph",
            "newly_mentioned_users": ['lmacken'],
            "diff": "I know the answer\n\nlololololol I do!     "
            "I swear.\n ...",
            "post": {
                "vote_up_count": 0,
                "text": "I know the answer\r\n\r\nlololololol I do!     "
                "I swear.",
                "summary": "I know the answer\n\nlololololol I do!     "
                "I swear.\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 5,
                "post_type": "answer"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.edit",
        "username": "threebean",
        "timestamp": 1359946482.179817
    }


class TestAskbotFlagOffensiveAdd(Base):
    """ Sent when a user flags an `Ask Fedora
    <https://ask.fedoraproject.org/questions>`_ question or answer
    as "offensive".
    """
    expected_title = "askbot.post.flag_offensive.add"
    expected_subti = "ralph flagged a question as offensive!"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/town',
        'tags/ohok',
        'threads/2',
    ])
    expected_link = "https://ask.fedoraproject.org/question/2/"
    msg = {
        "i": 1,
        "msg": {
            "instance": {
                "vote_up_count": 1,
                "text": "I think I know... totally.    "
                "asldkj for sure.",
                "summary": "I think I know... totally.    "
                "asldkj for sure.\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 3,
                "post_type": "question"
            },
            "agent": "ralph",
            "topmost_post_id": 2,
            "thread": {
                "tagnames": [
                    "town",
                    "ohok",
                ],
                "pk": 2,
                "title": "alskdjflaksjdf lakjsf a"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.flag_offensive.add",
        "username": "threebean",
        "timestamp": 1359947156.346592
    }


class TestAskbotFlagOffensiveRemove(Base):
    """ Sent when a user *un*flags an `Ask Fedora
    <https://ask.fedoraproject.org/questions>`_ question or answer
    as "offensive".
    """
    expected_title = "askbot.post.flag_offensive.delete"
    expected_subti = "ralph unflagged an answer as offensive..."
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/town',
        'tags/ohok',
        'threads/2',
    ])
    expected_link = "https://ask.fedoraproject.org/question/2/"
    msg = {
        "i": 1,
        "msg": {
            "instance": {
                "vote_up_count": 1,
                "text": "I think I know... totally.    "
                "asldkj for sure.",
                "summary": "I think I know... totally.    "
                "asldkj for sure.\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 3,
                "post_type": "answer"
            },
            "agent": "ralph",
            "topmost_post_id": 2,
            "thread": {
                "tagnames": [
                    "town",
                    "ohok",
                ],
                "pk": 2,
                "title": "alskdjflaksjdf lakjsf a"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.flag_offensive.delete",
        "username": "threebean",
        "timestamp": 1359947128.523792
    }


class TestAskbotUpdatedQuestion(Base):
    """ Messages get emitted on this topic anytime a question is updated.
    An 'update' includes a new question, a new answer, and a modification
    to either.  *This* example is one of a **question being modified** on
    `Ask Fedora <https://ask.fedoraproject.org/questions/>`_.
    """
    expected_title = "askbot.post.edit"
    expected_subti = "ralph updated the question 'alskdjflaksjdf lakjsf a'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_emails = {'ralph@fedoraproject.org': 'ralph'}
    expected_objects = set([
        'tags/town',
        'threads/2',
    ])
    expected_link = "https://ask.fedoraproject.org/question/2/"
    msg = {
        "i": 2,
        "msg": {
            "thread": {
                "tagnames": [
                    "town"
                ],
                "pk": 2,
                "title": "alskdjflaksjdf lakjsf a"
            },
            "created": False,
            "timestamp": 1359947640.0,
            "topmost_post_id": 2,
            "agent": "ralph",
            "newly_mentioned_users": [],
            "diff": "<p>alskdfj... the diff is actually here",
            "post": {
                "vote_up_count": 0,
                "text": "alskdfjalskdjf alkjasdalskdjf ...",
                "summary": "alskdfjalskdjf alkjasdalskdjf ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 2,
                "post_type": "question"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.edit",
        "username": "threebean",
        "timestamp": 1359947640.986208
    }


class TestAskbotUpdatedAnswer(Base):
    """ Messages get emitted on this topic anytime a question is updated.
    An 'update' includes a new question, a new answer, and a modification
    to either.  *This* example is one of a **answer being modified** on
    `Ask Fedora <https://ask.fedoraproject.org/questions/>`_.
    """
    expected_title = "askbot.post.edit"
    expected_subti = ("ralph updated an answer "
                      "to the question 'I have a new question'")
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'tags/lolol',
        'threads/3',
    ])
    expected_link = "https://ask.fedoraproject.org/question/3/"
    msg = {
        "i": 1,
        "msg": {
            "thread": {
                "tagnames": [
                    "lolol"
                ],
                "pk": 3,
                "title": "I have a new question"
            },
            "topmost_post_id": 3,
            "created": False,
            "timestamp": 1359947977.0,
            "agent": "ralph",
            "newly_mentioned_users": [],
            "diff": "<p>this is my test <del>answer</del><ins>answer."
            "ok?</ins></p>\n",
            "post": {
                "vote_up_count": 0,
                "text": "this is my test answer.    ok?",
                "summary": "this is my test answer.    ok?\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 6,
                "post_type": "answer"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.edit",
        "username": "threebean",
        "timestamp": 1359947978.125892
    }


class TestAskbotAnswerDeleted(Base):
    """ Messages with the ``askbot.post.delete`` topic get sent when either
    a question or an answer are deleted from `Ask Fedora
    <https://ask.fedoraproject.org/questions>`_.  The example here is one
    of an **answer** being deleted.
    """
    expected_title = "askbot.post.delete"
    expected_subti = "ralph deleted an answer on 'test 3 is a charm'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'threads/7',
    ])
    expected_link = "https://ask.fedoraproject.org/question/10/"
    msg = {
        "i": 1,
        "msg": {
            "topmost_post_id": 10,
            "instance": {
                "vote_up_count": 0,
                "text": "oh, ok.. coolio",
                "summary": "oh, ok.. coolio\n ...",
                "comment_count": 0,
                "vote_down_count": 0,
                "pk": 12,
                "post_type": "answer"
            },
            "agent": "ralph",
            "thread": {
                "tagnames": [
                    ""
                ],
                "pk": 7,
                "title": "test 3 is a charm"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.delete",
        "username": "threebean",
        "timestamp": 1359949257.459819
    }


class TestAskbotQuestionDeleted(Base):
    """ Messages with the ``askbot.post.delete`` topic get sent when either
    a question or an answer are deleted from `Ask Fedora
    <https://ask.fedoraproject.org/questions>`_.  The example here is one
    of an **question** being deleted.
    """
    expected_title = "askbot.post.delete"
    expected_subti = "ralph deleted the question 'test 3 is a charm'"
    expected_icon = "https://apps.fedoraproject.org/img/icons/ask_fedora.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'threads/7',
    ])
    expected_link = "https://ask.fedoraproject.org/question/10/"
    msg = {
        "i": 1,
        "msg": {
            "topmost_post_id": 10,
            "instance": {
                "vote_up_count": 0,
                "text": "this is a test message.",
                "summary": "this is a test message.\n ...",
                "comment_count": 1,
                "vote_down_count": 0,
                "pk": 10,
                "post_type": "question"
            },
            "agent": "ralph",
            "thread": {
                "tagnames": [
                    ""
                ],
                "pk": 7,
                "title": "test 3 is a charm"
            }
        },
        "topic": "org.fedoraproject.dev.askbot.post.delete",
        "username": "threebean",
        "timestamp": 1359949397.539748
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
config = dict(
    # Set this to dev if you're hacking on fedmsg or an app.
    # Set to stg or prod if running in the Fedora Infrastructure
    environment="dev",

    # Default is 0
    high_water_mark=0,
    io_threads=1,

    ## For the fedmsg-hub and fedmsg-relay. ##

    # We almost always want the fedmsg-hub to be sending messages with zmq as
    # opposed to amqp or stomp.
    zmq_enabled=True,

    # When subscribing to messages, we want to allow splats ('*') so we tell
    # the hub to not be strict when comparing messages topics to subscription
    # topics.
    zmq_strict=False,

    # Number of seconds to sleep after initializing waiting for sockets to
    # sync.
    post_init_sleep=0.2,
)



# This file is part of fedmsg.
# Copyright (C) 2015 Sayan Chowdhury.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Sayan Chowdhury <sayanchowdhury@fedoraproject.org>

from fedmsg_meta_fedora_infrastructure import BaseProcessor


class AutoCloudProcessor(BaseProcessor):

    __name__ = "autocloud"
    __description__ = "Automated Fedora Cloud Image Testing service"
    __link__ = "https://github.com/kushaldas/autocloud"
    __docs__ = "https://github.com/kushaldas/autocloud"
    __icon__ = "https://apps.fedoraproject.org/img/icons/fedimg.png"
    __obj__ = "Cloud Image Test"


    def _func_router(self, msg, fname, **config):
        """
        This method routes the messages based on the params and calls the
        appropriate method to process the message. The utility of the method is
        to cope up with the major message change during different releases.
        """
        FNAME = 'handle_%s_autocloud_%s'

        if ('compose_id' in msg['msg'] or 'compose_job_id' in msg['msg'] or
                'autocloud.compose' in msg['topic']):
            return getattr(self, FNAME % ('v2', fname))(msg, **config)
        else:
            return getattr(self, FNAME % ('v1', fname))(msg, **config)

    def subtitle(self, msg, **config):
        return self._func_router(msg, 'subtitle', **config)

    def secondary_icon(self, msg, **config):
        return self.__icon__

    def link(self, msg, **config):
        return self._func_router(msg, 'link', **config)

    def objects(self, msg, **config):
        return self._func_router(msg, 'objects', **config)

    def handle_v2_autocloud_subtitle(self, msg, **config):
        status = msg['msg']['status']
        if 'autocloud.image' in msg['topic']:
            compose_id = msg['msg']['compose_id']
            image_name = msg['msg']['image_name']
            release = msg['msg'].get('release')

            if status == "queued":
                tmpl = self._("{image_name}({compose_id}) is {status} for "
                                "testing")
            if status == "running":
                tmpl = self._("The tests for the {image_name}({compose_id}) "
                                "have started {status}")
            if status == "aborted":
                tmpl = self._("The tests for the {image_name}({compose_id}) "
                                "have been {status}")
            if status == "failed":
                tmpl = self._("The tests for the {image_name}({compose_id}) "
                                "{status}")
            if status == "success":
                tmpl = self._("The tests for {image_name}({compose_id}) were a"
                                " {status}")

            if release:
                image_name = "%s (%s)" % (image_name, release)

            return tmpl.format(image_name=image_name, status=status,
                                compose_id=compose_id)

        if "autocloud.compose" in msg['topic']:
            compose_id = msg['msg'].get(
                    'id', msg['msg'].get('compose_id', 'A compose'))

            if status == 'queued':
                tmpl = self._("{compose_id} is {status} for testing")
            if status == 'running':
                tmpl = self._("{compose_id} tests have started {status}")
            if status == 'completed':
                tmpl = self._("{compose_id} tests have {status}")

            return tmpl.format(compose_id=compose_id, status=status)

    def handle_v2_autocloud_link(self, msg, **config):
        if 'autocloud.image' in msg['topic']:
            job_id = msg['msg'].get('job_id')
            if job_id:
                template = 'https://apps.fedoraproject.org/autocloud/jobs/' + \
                    '%s/output'
                return template % job_id
            else:
                return msg['msg']['image_url']
        elif 'autocloud.compose' in msg['topic']:
            compose_job_id = msg['msg'].get('compose_job_id')
            if compose_job_id is not None:
                return 'https://apps.fedoraproject.org/autocloud/jobs/%s' % (
                    compose_job_id)
        return 'https://apps.fedoraproject.org/autocloud/'

    def handle_v2_autocloud_objects(self, msg, **config):
        status = msg['msg']['status']
        if 'autocloud.image' in msg['topic']:
            return set(['autocloud/image/' + status])
        elif 'autocloud.compose' in msg['topic']:
            return set(['autocloud/compose/' + status])

    def handle_v1_autocloud_subtitle(self, msg, **config):
        image_name = msg['msg']['image_name']
        status = msg['msg']['status']
        release = msg['msg'].get('release')
        if 'autocloud.image' in msg['topic']:
            if status == "queued":
                tmpl = self._("{image_name} is {status} for testing")
            if status == "running":
                tmpl = self._("The tests for the {image_name} have "
                              "started {status}")
            if status == "aborted":
                tmpl = self._("The tests for the {image_name} have "
                              "been {status}")
            if status == "failed":
                tmpl = self._("The tests for the {image_name} {status}")
            if status == "success":
                tmpl = self._("The tests for {image_name} were a {status}")

            if release:
                image_name = "%s (%s)" % (image_name, release)

            return tmpl.format(image_name=image_name, status=status)

    def handle_v1_autocloud_link(self, msg, **config):
        job_id = msg['msg'].get('job_id')
        if job_id:
            template = 'https://apps.fedoraproject.org/autocloud/jobs/%s/output'
            return template % job_id
        else:
            return msg['msg']['image_url']

    def handle_v1_autocloud_objects(self,msg, **config):
        status = msg['msg']['status']
        return set(['autocloud/image/' + status])





# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for ftpsync messages """

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestFTPSyncAtomic(Base):
    """ These messages are published when a new compose of Fedora Atomic
    is synced out to the master mirror.
    """
    expected_title = "bodhi.updates.fedora.sync"
    expected_subti = "New Fedora 21 atomic content synced out " + \
        "(60.38M changed with 0 files deleted)"
    expected_link = \
        "https://download.fedoraproject.org/pub/fedora/linux/atomic/21/"
    expected_objects = set(['fedora/atomic/21'])

    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.bodhi.updates.fedora.sync",
        "msg": {
            "bytes": "60.38M",
            "deleted": "0",
            "repo": "atomic",
            "release": "21",
        },
    }


class TestFTPSyncFedora(Base):
    """ These messages are published when new updates (fresh out of the "mash"
    process) are synced out to the master mirror.

    Here's an example for the fedora 20 stable repos:
    """
    expected_title = "bodhi.updates.fedora.sync"
    expected_subti = "New Fedora 20 updates content synced out " + \
        "(2.8M changed with 0 files deleted)"
    expected_link = \
        "https://download.fedoraproject.org/pub/fedora/linux/updates/20/"
    expected_objects = set(['fedora/updates/20'])

    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.bodhi.updates.fedora.sync",
        "msg": {
            "bytes": "2.8M",
            "deleted": "0",
            "repo": "updates",
            "release": "20",
        },
    }


class TestFTPSyncEPELTesting(Base):
    """ These messages are published when new updates (fresh out of the "mash"
    process) are synced out to the master mirror.

    Here's an example for the epel 6 testing repos:
    """
    expected_title = "bodhi.updates.epel.sync"
    expected_subti = "New EPEL 6 epel-testing content synced out " + \
        "(28493k changed with 0 files deleted)"
    expected_link = \
        "https://download.fedoraproject.org/pub/epel/testing/6/"
    expected_objects = set(['epel/epel-testing/6'])

    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.bodhi.updates.epel.sync",
        "msg": {
            "bytes": "28493k",
            "deleted": "0",
            "repo": "epel-testing",
            "release": "6",
        },
    }


add_doc(locals())



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class RelengProcessor(BaseProcessor):
    __name__ = "releng"
    __description__ = "Fedora Release Engineering"
    __link__ = "https://git.fedorahosted.org/cgit/releng"
    __docs__ = "https://fedoraproject.org/wiki/ReleaseEngineering"
    __obj__ = "Releng Events"
    __icon__ = "https://apps.fedoraproject.org/img/icons/atomic.png"

    def subtitle(self, msg, **config):
        release = msg['msg']['atomic_raw'].get('release', '')
        if msg['topic'].endswith('.releng.atomic.twoweek.begin'):
            tmpl = self._("Release engineering scripts started evaluating a "
                          "new set of builds for a Fedora {release} Atomic "
                          "Host release")
            return tmpl.format(release=release)
        elif msg['topic'].endswith('.releng.atomic.twoweek.complete'):
            tmpl = self._("A new release of Fedora {release} Atomic "
                          "Host is ready")
            return tmpl.format(release=release)
        else:
            return None

    def link(self, msg, **config):
        return "https://download.fedoraproject.org/pub/alt/atomic/stable/"

    def secondary_icon(self, msg, **config):
        return self.__icon__

    def objects(self, msg, **config):
        return set([
            "%s/%s" % (msg['msg'][key]['release'], key)
            for key in msg['msg'] if key.startswith('atomic_')
        ])



# This file is part of fedmsg.
# Copyright (C) 2016 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for PDC messages. """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from fedmsg_meta_fedora_infrastructure.tests.common import add_doc


class TestPDCComposeImport(Base):
    """ The `Product Definition Center <https://pdc.fedoraproject.org>`_
    publishes messages on this topic whenever a new compose entry is created.
    This message comes early, when *we first begin to import the compose*.
    There are other messages that come after, when the images and rpms for the
    compose finish being imported.
    """
    expected_title = "pdc.compose"
    expected_subti = ("An entry for the Fedora-Rawhide-20160331.n.0 nightly "
                      "compose was created in the Product Definition Center")
    expected_link = ("https://pdc.fedoraproject.org/rest_api/v1/"
                     "composes/Fedora-Rawhide-20160331.n.0/")
    expected_objects = set(['Fedora-Rawhide-20160331.n.0/create/compose'])
    msg = {
        "timestamp": 1459420734.0,
        "msg_id": "2016-e9ec264b-ce44-4260-8641-e385e71da171",
        "topic": "org.fedoraproject.prod.pdc.compose",
        "source_version": "0.6.5",
        "msg": {
            "action": "create",
            "compose_respin": 0,
            "compose_date": "2016-03-31",
            "compose_id": "Fedora-Rawhide-20160331.n.0",
            "compose_type": "nightly"
        }
    }


class TestPDCImagesImport(Base):
    """ The `Product Definition Center <https://pdc.fedoraproject.org>`_
    publishes messages on this topic whenever a set of images are uploaded for
    a new compose entry. This message doesn't necessarily signify that the
    compose is fully imported.  The `pdc-updater
    <https://github.com/fedora-infra/pdc-updater>`_ project typically import
    the images first and the rpms second, but it could be that an admin
    manually imports a compose in which case the order is not guaranteed.
    """
    expected_title = "pdc.images"
    expected_subti = ("PDC imported metadata for 3 images for "
                      "the Fedora-Rawhide-20160331.n.0 nightly compose")
    expected_link = ("https://pdc.fedoraproject.org/rest_api/v1/"
                     "composes/Fedora-Rawhide-20160331.n.0/")
    expected_objects = set(['Fedora-Rawhide-20160331.n.0/import/images'])
    msg = {
        "timestamp": 1459420734.0,
        "topic": "org.fedoraproject.prod.pdc.images",
        "source_version": "0.6.5",
        "msg": {
            "action": "import",
            "attribute": "images",
            "count": 3,
            "compose_respin": 0,
            "compose_date": "2016-03-31",
            "compose_id": "Fedora-Rawhide-20160331.n.0",
            "compose_type": "nightly"
        }
    }


class TestPDCRPMsImport(Base):
    """ The `Product Definition Center <https://pdc.fedoraproject.org>`_
    publishes messages on this topic whenever a set of rpms are uploaded for
    a new compose entry. This message doesn't necessarily signify that the
    compose is fully imported.  The `pdc-updater
    <https://github.com/fedora-infra/pdc-updater>`_ project typically import
    the images first and the rpms second, but it could be that an admin
    manually imports a compose in which case the order is not guaranteed.
    """
    expected_title = "pdc.rpms"
    expected_subti = ("PDC imported metadata for 207 rpms for "
                      "the Fedora-Rawhide-20160331.n.0 nightly compose")
    expected_link = ("https://pdc.fedoraproject.org/rest_api/v1/"
                     "composes/Fedora-Rawhide-20160331.n.0/")
    expected_objects = set(['Fedora-Rawhide-20160331.n.0/import/rpms'])
    msg = {
        "timestamp": 1459420734.0,
        "topic": "org.fedoraproject.prod.pdc.rpms",
        "source_version": "0.6.5",
        "msg": {
            "action": "import",
            "attribute": "rpms",
            "count": 207,
            "compose_respin": 0,
            "compose_date": "2016-03-31",
            "compose_id": "Fedora-Rawhide-20160331.n.0",
            "compose_type": "nightly"
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class NagiosProcessor(BaseProcessor):
    __name__ = "Nagios"
    __description__ = "Fedora Infra Service Monitoring"
    __link__ = "https://admin.fedoraproject.org/nagios"
    __docs__ = "https://infrastructure.fedoraproject.org/infra/docs/nagios.rst"
    __obj__ = "Service Outage Alerts"
    __icon__ = "https://apps.fedoraproject.org/img/icons/nagios-logo.png"

    def subtitle(self, msg, **config):
        states = {
            'CRITICAL': self._('down'),
            'WARNING': self._('having problems'),
            'OK': self._('back up'),
        }
        state = msg['msg']['state']
        state = states.get(state, self._('in an unknown state'))
        host = msg['msg']['host']

        result = None
        if msg['topic'].endswith('host.state.change'):
            tmpl = self._('{host} is {state}')
            result = tmpl.format(host=host, state=state)
        elif msg['topic'].endswith('service.state.change'):
            service = msg['msg'].get('service', 'some service')
            tmpl = self._('{service} is {state} on {host}')
            result = tmpl.format(service=service, host=host, state=state)

        if msg['msg']['type'] == 'ACKNOWLEDGEMENT':
            users = self.usernames(msg, **config)
            agent = 'somebody'
            if users:
                agent = list(users)[0]
            prefix = self._('{agent} acknowledged that ').format(agent=agent)
            result = prefix + result

        if result and 'output' in msg['msg']:
            result += ': "{output}"'.format(output=msg['msg']['output'])

        return result

    def link(self, msg, **config):
        base = 'https://admin.fedoraproject.org/nagios/cgi-bin/status.cgi'
        query = '?navbarsearch=1&host='
        host = msg['msg']['host']
        return base + query + host

    def secondary_icon(self, msg, **config):
        users = self.usernames(msg, **config)
        if users:
            return avatar_url(list(users)[0])
        return self.__icon__

    def usernames(self, msg, **config):
        users = set()
        service_ack = msg['msg']['service_ack_author']
        if service_ack:
            users.add(service_ack.split('://')[-1].split('.')[0])

        host_ack = msg['msg']['host_ack_author']
        if host_ack:
            users.add(host_ack.split('://')[-1].split('.')[0])

        return users

    def objects(self, msg, **config):
        host = msg['msg']['host']
        service = msg['msg'].get('service', 'unknown')
        if msg['topic'].endswith('host.state.change'):
            service = 'state'
        return set([
            '/'.join([host, service]),
        ])
